
class Declaration;

/** Interface for HILTI declarations. */
class Concept : public hilti::util::type_erasure::ConceptBase {
public:
    Concept() = default;
    Concept(const Concept&) = default;
    Concept(Concept&&) = default;
    virtual ~Concept() = default;
    /** Returns the declaration's ID. */
    virtual ID id() const = 0;
    /** Returns the declaration's linkage. */
    virtual Linkage linkage() const = 0;
    /**
* Returns true if one cannot assign to what this declaration defines.
*/
    virtual bool isConstant() const = 0;
    /**
* Returns a user-friendly name for the type of object the declararion
* refers to. This is used in error messages.
*/
    virtual std::string displayName() const = 0;
    /** Returns true if the declaration is equivalent to another one in HILTI semantics. */
    virtual bool isEqual(const Declaration& other) const = 0;
    /** Implements the `Node` interface. */
    virtual hilti::node::Properties properties() const = 0;
    /** Implements the `Node` interface. */
    virtual const std::vector<hilti::Node>& childs() const = 0;
    /** Implements the `Node` interface. */
    virtual std::vector<hilti::Node>& childs()  = 0;
    /** Implements the `Node` interface. */
    virtual const Meta& meta() const = 0;
    /** Implements the `Node` interface. */
    virtual void setMeta(Meta m)  = 0;
    /** Implements the `Node` interface. */
    virtual const NodeRef& originalNode() const = 0;
    /** Implements the `Node` interface. */
    virtual void setOriginalNode(const NodeRef& n)  = 0;
    /** Implements the `Node` interface. */
    virtual void clearCache()  = 0;
    Concept& operator=(const Concept&) = default;
    Concept& operator=(Concept&&) = default;
    virtual Declaration _clone() const = 0;
    virtual hilti::IntrusivePtr<Concept> _clone_ptr() const = 0;
private:
    
};

template<typename T>
using ModelBase = hilti::util::type_erasure::ModelBase<T, Concept>;

template<class T>
class Model : public ModelBase<T> {
public:
    using ModelBase<T>::ModelBase;
    using ModelBase<T>::data;

    ID id() const final;
    Linkage linkage() const final;
    bool isConstant() const final;
    std::string displayName() const final;
    bool isEqual(const Declaration& other) const final;
    hilti::node::Properties properties() const final;
    const std::vector<hilti::Node>& childs() const final;
    std::vector<hilti::Node>& childs()  final;
    const Meta& meta() const final;
    void setMeta(Meta m)  final;
    const NodeRef& originalNode() const final;
    void setOriginalNode(const NodeRef& n)  final;
    void clearCache()  final;
    Declaration _clone() const final;
    hilti::IntrusivePtr<Concept> _clone_ptr() const final;
};

using ErasedBase = hilti::util::type_erasure::ErasedBase<trait::isDeclaration, Concept, Model>;

class Declaration : public ErasedBase, public trait::isNode {
public:
    using ErasedBase::ErasedBase;

    Declaration() noexcept = default;
    Declaration(Declaration&&) noexcept = default;
    Declaration& operator=(Declaration&&) = default;
    ~Declaration() override = default;
    Declaration(const Declaration& other) = default;
    Declaration& operator=(const Declaration& other) = default;

    ID id() const;
    Linkage linkage() const;
    bool isConstant() const;
    std::string displayName() const;
    bool isEqual(const Declaration& other) const;
    hilti::node::Properties properties() const;
    const std::vector<hilti::Node>& childs() const;
    std::vector<hilti::Node>& childs() ;
    const Meta& meta() const;
    void setMeta(Meta m) ;
    const NodeRef& originalNode() const;
    void setOriginalNode(const NodeRef& n) ;
    void clearCache() ;
    Declaration _clone() const;

    hilti::IntrusivePtr<Concept> _clone_ptr() const;
};

template<typename T>
ID Model<T>::id() const { return data().id(); }

template<typename T>
Linkage Model<T>::linkage() const { return data().linkage(); }

template<typename T>
bool Model<T>::isConstant() const { return data().isConstant(); }

template<typename T>
std::string Model<T>::displayName() const { return data().displayName(); }

template<typename T>
bool Model<T>::isEqual(const Declaration& other) const { return data().isEqual(other); }

template<typename T>
hilti::node::Properties Model<T>::properties() const { return data().properties(); }

template<typename T>
const std::vector<hilti::Node>& Model<T>::childs() const { return data().childs(); }

template<typename T>
std::vector<hilti::Node>& Model<T>::childs()  { return data().childs(); }

template<typename T>
const Meta& Model<T>::meta() const { return data().meta(); }

template<typename T>
void Model<T>::setMeta(Meta m)  { return data().setMeta(m); }

template<typename T>
const NodeRef& Model<T>::originalNode() const { return data().originalNode(); }

template<typename T>
void Model<T>::setOriginalNode(const NodeRef& n)  { return data().setOriginalNode(n); }

template<typename T>
void Model<T>::clearCache()  { return data().clearCache(); }

template<typename T>
Declaration Model<T>::_clone() const { return T(data()); }

template<typename T>
hilti::IntrusivePtr<Concept> Model<T>::_clone_ptr() const {
    if constexpr ( std::is_base_of<hilti::util::type_erasure::trait::Singleton, T>::value )
        return nullptr;
    else
        return hilti::make_intrusive<Model>(T(data()));
}

inline ID Declaration::id() const { return data()->id(); }

inline Linkage Declaration::linkage() const { return data()->linkage(); }

inline bool Declaration::isConstant() const { return data()->isConstant(); }

inline std::string Declaration::displayName() const { return data()->displayName(); }

inline bool Declaration::isEqual(const Declaration& other) const { return data()->isEqual(other); }

inline hilti::node::Properties Declaration::properties() const { return data()->properties(); }

inline const std::vector<hilti::Node>& Declaration::childs() const { return data()->childs(); }

inline std::vector<hilti::Node>& Declaration::childs()  { return data()->childs(); }

inline const Meta& Declaration::meta() const { return data()->meta(); }

inline void Declaration::setMeta(Meta m)  { return data()->setMeta(std::move(m)); }

inline const NodeRef& Declaration::originalNode() const { return data()->originalNode(); }

inline void Declaration::setOriginalNode(const NodeRef& n)  { return data()->setOriginalNode(n); }

inline void Declaration::clearCache()  { return data()->clearCache(); }

inline Declaration Declaration::_clone() const { return data() ? data()->_clone() : Declaration(); }

inline hilti::IntrusivePtr<Concept> Declaration::_clone_ptr() const {
    if ( data() )
        return data()->_clone_ptr();
    else
        return nullptr;
}



