
class Statement;

/** Interface for HILTI statements. */
class Concept : public hilti::util::type_erasure::ConceptBase {
public:
    Concept() = default;
    Concept(const Concept&) = default;
    Concept(Concept&&) = default;
    virtual ~Concept() = default;
    /** Returns true if the statement is equivalent to another one in HILTI semantics. */
    virtual bool isEqual(const Statement& other) const = 0;
    /** Implements the `Node` interface. */
    virtual hilti::node::Properties properties() const = 0;
    /** Implements the `Node` interface. */
    virtual const std::vector<hilti::Node>& childs() const = 0;
    /** Implements the `Node` interface. */
    virtual std::vector<hilti::Node>& childs()  = 0;
    /** Implements the `Node` interface. */
    virtual const Meta& meta() const = 0;
    /** Implements the `Node` interface. */
    virtual void setMeta(Meta m)  = 0;
    /** Implements the `Node` interface. */
    virtual const NodeRef& originalNode() const = 0;
    /** Implements the `Node` interface. */
    virtual void setOriginalNode(const NodeRef& n)  = 0;
    /** Implements the `Node` interface. */
    virtual void clearCache()  = 0;
    Concept& operator=(const Concept&) = default;
    Concept& operator=(Concept&&) = default;
    virtual Statement _clone() const = 0;
    virtual hilti::IntrusivePtr<Concept> _clone_ptr() const = 0;
private:
    
};

template<typename T>
using ModelBase = hilti::util::type_erasure::ModelBase<T, Concept>;

template<class T>
class Model : public ModelBase<T> {
public:
    using ModelBase<T>::ModelBase;
    using ModelBase<T>::data;

    bool isEqual(const Statement& other) const final;
    hilti::node::Properties properties() const final;
    const std::vector<hilti::Node>& childs() const final;
    std::vector<hilti::Node>& childs()  final;
    const Meta& meta() const final;
    void setMeta(Meta m)  final;
    const NodeRef& originalNode() const final;
    void setOriginalNode(const NodeRef& n)  final;
    void clearCache()  final;
    Statement _clone() const final;
    hilti::IntrusivePtr<Concept> _clone_ptr() const final;
};

using ErasedBase = hilti::util::type_erasure::ErasedBase<trait::isStatement, Concept, Model>;

class Statement : public ErasedBase, public trait::isNode {
public:
    using ErasedBase::ErasedBase;

    Statement() noexcept = default;
    Statement(Statement&&) noexcept = default;
    Statement& operator=(Statement&&) = default;
    ~Statement() override = default;
    Statement(const Statement& other) = default;
    Statement& operator=(const Statement& other) = default;

    bool isEqual(const Statement& other) const;
    hilti::node::Properties properties() const;
    const std::vector<hilti::Node>& childs() const;
    std::vector<hilti::Node>& childs() ;
    const Meta& meta() const;
    void setMeta(Meta m) ;
    const NodeRef& originalNode() const;
    void setOriginalNode(const NodeRef& n) ;
    void clearCache() ;
    Statement _clone() const;

    hilti::IntrusivePtr<Concept> _clone_ptr() const;
};

template<typename T>
bool Model<T>::isEqual(const Statement& other) const { return data().isEqual(other); }

template<typename T>
hilti::node::Properties Model<T>::properties() const { return data().properties(); }

template<typename T>
const std::vector<hilti::Node>& Model<T>::childs() const { return data().childs(); }

template<typename T>
std::vector<hilti::Node>& Model<T>::childs()  { return data().childs(); }

template<typename T>
const Meta& Model<T>::meta() const { return data().meta(); }

template<typename T>
void Model<T>::setMeta(Meta m)  { return data().setMeta(m); }

template<typename T>
const NodeRef& Model<T>::originalNode() const { return data().originalNode(); }

template<typename T>
void Model<T>::setOriginalNode(const NodeRef& n)  { return data().setOriginalNode(n); }

template<typename T>
void Model<T>::clearCache()  { return data().clearCache(); }

template<typename T>
Statement Model<T>::_clone() const { return T(data()); }

template<typename T>
hilti::IntrusivePtr<Concept> Model<T>::_clone_ptr() const {
    if constexpr ( std::is_base_of<hilti::util::type_erasure::trait::Singleton, T>::value )
        return nullptr;
    else
        return hilti::make_intrusive<Model>(T(data()));
}

inline bool Statement::isEqual(const Statement& other) const { return data()->isEqual(other); }

inline hilti::node::Properties Statement::properties() const { return data()->properties(); }

inline const std::vector<hilti::Node>& Statement::childs() const { return data()->childs(); }

inline std::vector<hilti::Node>& Statement::childs()  { return data()->childs(); }

inline const Meta& Statement::meta() const { return data()->meta(); }

inline void Statement::setMeta(Meta m)  { return data()->setMeta(std::move(m)); }

inline const NodeRef& Statement::originalNode() const { return data()->originalNode(); }

inline void Statement::setOriginalNode(const NodeRef& n)  { return data()->setOriginalNode(n); }

inline void Statement::clearCache()  { return data()->clearCache(); }

inline Statement Statement::_clone() const { return data() ? data()->_clone() : Statement(); }

inline hilti::IntrusivePtr<Concept> Statement::_clone_ptr() const {
    if ( data() )
        return data()->_clone_ptr();
    else
        return nullptr;
}



