
class Type;

/** Interface for HILTI types. */
class Concept : public hilti::util::type_erasure::ConceptBase {
public:
    Concept() = default;
    Concept(const Concept&) = default;
    Concept(Concept&&) = default;
    virtual ~Concept() = default;
    /** Returns true if the type is equivalent to another HILTI type. */
    virtual bool isEqual(const Type& other) const = 0;
    /**
* Returns any parameters associated with type. If a type is declared as
* `T<A,B,C>` this returns a vector of the AST nodes for `A`, `B`, and
* `C`.
*/
    virtual std::vector<Node> typeParameters() const = 0;
    /**
* Returns true if all instances of the same type class can be coerced
* into the current instance, independent of their pararameters. In HILTI
* source code, this typically corresponds to a type `T<*>`.
*/
    virtual bool isWildcard() const = 0;
    /** Returns the set of flags associated with the type. */
    virtual type::Flags flags() const = 0;
    /** Returns true if a given type flag has been set. */
    virtual bool hasFlag(type::Flag f) const = 0;
    /** Returns the type of an iterator for this type. */
    virtual Type iteratorType(bool const_) const = 0;
    /** Returns the type of an view for this type. */
    virtual Type viewType() const = 0;
    /** Returns the type of elements the iterator traveres. */
    virtual Type dereferencedType() const = 0;
    /** Returns the type of elements the container stores. */
    virtual Type elementType() const = 0;
    /**
* Returns the fully deferenced type that this type corresponds to. This,
* for example, follows a type ID to the actual type it maps to.
*/
    virtual Type effectiveType() const = 0;
    /** For internal use. Use ``type::isConstant()` instead. */
    virtual bool _isConstant() const = 0;
    /** For internal use. */
    virtual type::detail::State& _state()  = 0;
    /**
* Returns a HILTI ID associated with the type if any has been set. This
* method is implemented for all types in `TypeBase`.
*/
    virtual std::optional<ID> typeID() const = 0;
    /**
* Returns a C++ ID associated with the type if any has been set. This
* method is implemented for all types in `TypeBase`.
*/
    virtual std::optional<ID> cxxID() const = 0;
    /** Implements the `Node` interface. */
    virtual hilti::node::Properties properties() const = 0;
    /** Implements the `Node` interface. */
    virtual const std::vector<hilti::Node>& childs() const = 0;
    /** Implements the `Node` interface. */
    virtual std::vector<hilti::Node>& childs()  = 0;
    /** Implements the `Node` interface. */
    virtual const Meta& meta() const = 0;
    /** Implements the `Node` interface. */
    virtual void setMeta(Meta m)  = 0;
    /** Implements the `Node` interface. */
    virtual const NodeRef& originalNode() const = 0;
    /** Implements the `Node` interface. */
    virtual void setOriginalNode(const NodeRef& n)  = 0;
    /** Implements the `Node` interface. */
    virtual void clearCache()  = 0;
    virtual bool _isAllocable() const = 0;
    virtual bool _isDereferencable() const = 0;
    virtual bool _isIterable() const = 0;
    virtual bool _isViewable() const = 0;
    virtual bool _isIterator() const = 0;
    virtual bool _isView() const = 0;
    virtual bool _isParameterized() const = 0;
    virtual bool _isReferenceType() const = 0;
    virtual bool _isMutable() const = 0;
    virtual bool _hasDynamicType() const = 0;
    virtual bool _isRuntimeNonTrivial() const = 0;
    virtual bool _isOnHeap() const = 0;
    Concept& operator=(const Concept&) = default;
    Concept& operator=(Concept&&) = default;
    virtual Type _clone() const = 0;
    virtual hilti::IntrusivePtr<Concept> _clone_ptr() const = 0;
private:
    
};

template<typename T>
using ModelBase = hilti::util::type_erasure::ModelBase<T, Concept>;

template<class T>
class Model : public ModelBase<T> {
public:
    using ModelBase<T>::ModelBase;
    using ModelBase<T>::data;

    bool isEqual(const Type& other) const final;
    std::vector<Node> typeParameters() const final;
    bool isWildcard() const final;
    type::Flags flags() const final;
    bool hasFlag(type::Flag f) const final;
    Type iteratorType(bool const_) const final;
    Type viewType() const final;
    Type dereferencedType() const final;
    Type elementType() const final;
    Type effectiveType() const final;
    bool _isConstant() const final;
    type::detail::State& _state()  final;
    std::optional<ID> typeID() const final;
    std::optional<ID> cxxID() const final;
    hilti::node::Properties properties() const final;
    const std::vector<hilti::Node>& childs() const final;
    std::vector<hilti::Node>& childs()  final;
    const Meta& meta() const final;
    void setMeta(Meta m)  final;
    const NodeRef& originalNode() const final;
    void setOriginalNode(const NodeRef& n)  final;
    void clearCache()  final;
    bool _isAllocable() const final;
    bool _isDereferencable() const final;
    bool _isIterable() const final;
    bool _isViewable() const final;
    bool _isIterator() const final;
    bool _isView() const final;
    bool _isParameterized() const final;
    bool _isReferenceType() const final;
    bool _isMutable() const final;
    bool _hasDynamicType() const final;
    bool _isRuntimeNonTrivial() const final;
    bool _isOnHeap() const final;
    Type _clone() const final;
    hilti::IntrusivePtr<Concept> _clone_ptr() const final;
};

using ErasedBase = hilti::util::type_erasure::ErasedBase<hilti::trait::isType, Concept, Model>;

class Type : public ErasedBase, public hilti::trait::isNode {
public:
    using ErasedBase::ErasedBase;

    Type() noexcept = default;
    Type(Type&&) noexcept = default;
    Type& operator=(Type&&) = default;
    ~Type() override = default;
    Type(const Type& other) = default;
    Type& operator=(const Type& other) = default;

    bool isEqual(const Type& other) const;
    std::vector<Node> typeParameters() const;
    bool isWildcard() const;
    type::Flags flags() const;
    bool hasFlag(type::Flag f) const;
    Type iteratorType(bool const_) const;
    Type viewType() const;
    Type dereferencedType() const;
    Type elementType() const;
    Type effectiveType() const;
    bool _isConstant() const;
    type::detail::State& _state() ;
    std::optional<ID> typeID() const;
    std::optional<ID> cxxID() const;
    hilti::node::Properties properties() const;
    const std::vector<hilti::Node>& childs() const;
    std::vector<hilti::Node>& childs() ;
    const Meta& meta() const;
    void setMeta(Meta m) ;
    const NodeRef& originalNode() const;
    void setOriginalNode(const NodeRef& n) ;
    void clearCache() ;
    bool _isAllocable() const;
    bool _isDereferencable() const;
    bool _isIterable() const;
    bool _isViewable() const;
    bool _isIterator() const;
    bool _isView() const;
    bool _isParameterized() const;
    bool _isReferenceType() const;
    bool _isMutable() const;
    bool _hasDynamicType() const;
    bool _isRuntimeNonTrivial() const;
    bool _isOnHeap() const;
    Type _clone() const;

    hilti::IntrusivePtr<Concept> _clone_ptr() const;
};

template<typename T>
bool Model<T>::isEqual(const Type& other) const { return data().isEqual(other); }

template<typename T>
std::vector<Node> Model<T>::typeParameters() const { 
    if constexpr ( std::is_base_of<hilti::type::trait::isParameterized, T>::value )
        return data().typeParameters();
    else
        return {};
 }

template<typename T>
bool Model<T>::isWildcard() const { 
    if constexpr ( std::is_base_of<hilti::type::trait::isParameterized, T>::value )
        return data().isWildcard();
    else
        return false;
 }

template<typename T>
type::Flags Model<T>::flags() const { return data().flags(); }

template<typename T>
bool Model<T>::hasFlag(type::Flag f) const { return data().hasFlag(f); }

template<typename T>
Type Model<T>::iteratorType(bool const_) const { 
    if constexpr ( std::is_base_of<hilti::type::trait::isIterable, T>::value )
        return data().iteratorType(const_);
    else if constexpr ( std::is_base_of<hilti::type::trait::isView, T>::value )
        return data().iteratorType(const_);
    else
        throw std::bad_cast();
 }

template<typename T>
Type Model<T>::viewType() const { 
    if constexpr ( std::is_base_of<hilti::type::trait::isViewable, T>::value )
        return data().viewType();
    else
        throw std::bad_cast();
 }

template<typename T>
Type Model<T>::dereferencedType() const { 
    if constexpr ( std::is_base_of<hilti::type::trait::isDereferencable, T>::value )
        return data().dereferencedType();
    else
        throw std::bad_cast();
 }

template<typename T>
Type Model<T>::elementType() const { 
    if constexpr ( std::is_base_of<hilti::type::trait::isIterable, T>::value )
        return data().elementType();
    else
        throw std::bad_cast();
 }

template<typename T>
Type Model<T>::effectiveType() const { 
    if constexpr ( std::is_base_of<hilti::type::trait::hasDynamicType, T>::value )
        return data().effectiveType();
    else
        throw std::bad_cast();
 }

template<typename T>
bool Model<T>::_isConstant() const { return data()._isConstant(); }

template<typename T>
type::detail::State& Model<T>::_state()  { return data()._state(); }

template<typename T>
std::optional<ID> Model<T>::typeID() const { return data().typeID(); }

template<typename T>
std::optional<ID> Model<T>::cxxID() const { return data().cxxID(); }

template<typename T>
hilti::node::Properties Model<T>::properties() const { return data().properties(); }

template<typename T>
const std::vector<hilti::Node>& Model<T>::childs() const { return data().childs(); }

template<typename T>
std::vector<hilti::Node>& Model<T>::childs()  { return data().childs(); }

template<typename T>
const Meta& Model<T>::meta() const { return data().meta(); }

template<typename T>
void Model<T>::setMeta(Meta m)  { return data().setMeta(m); }

template<typename T>
const NodeRef& Model<T>::originalNode() const { return data().originalNode(); }

template<typename T>
void Model<T>::setOriginalNode(const NodeRef& n)  { return data().setOriginalNode(n); }

template<typename T>
void Model<T>::clearCache()  { return data().clearCache(); }

template<typename T>
bool Model<T>::_isAllocable() const { return std::is_base_of<hilti::type::trait::isAllocable, T>::value; }

template<typename T>
bool Model<T>::_isDereferencable() const { return std::is_base_of<hilti::type::trait::isDereferencable, T>::value; }

template<typename T>
bool Model<T>::_isIterable() const { return std::is_base_of<hilti::type::trait::isIterable, T>::value; }

template<typename T>
bool Model<T>::_isViewable() const { return std::is_base_of<hilti::type::trait::isViewable, T>::value; }

template<typename T>
bool Model<T>::_isIterator() const { return std::is_base_of<hilti::type::trait::isIterator, T>::value; }

template<typename T>
bool Model<T>::_isView() const { return std::is_base_of<hilti::type::trait::isView, T>::value; }

template<typename T>
bool Model<T>::_isParameterized() const { return std::is_base_of<hilti::type::trait::isParameterized, T>::value; }

template<typename T>
bool Model<T>::_isReferenceType() const { return std::is_base_of<hilti::type::trait::isReferenceType, T>::value; }

template<typename T>
bool Model<T>::_isMutable() const { return std::is_base_of<hilti::type::trait::isMutable, T>::value; }

template<typename T>
bool Model<T>::_hasDynamicType() const { return std::is_base_of<hilti::type::trait::hasDynamicType, T>::value; }

template<typename T>
bool Model<T>::_isRuntimeNonTrivial() const { return std::is_base_of<hilti::type::trait::isRuntimeNonTrivial, T>::value; }

template<typename T>
bool Model<T>::_isOnHeap() const { return std::is_base_of<hilti::type::trait::isOnHeap, T>::value; }

template<typename T>
Type Model<T>::_clone() const { return T(data()); }

template<typename T>
hilti::IntrusivePtr<Concept> Model<T>::_clone_ptr() const {
    if constexpr ( std::is_base_of<hilti::util::type_erasure::trait::Singleton, T>::value )
        return nullptr;
    else
        return hilti::make_intrusive<Model>(T(data()));
}

inline bool Type::isEqual(const Type& other) const { return data()->isEqual(other); }

inline std::vector<Node> Type::typeParameters() const { return data()->typeParameters(); }

inline bool Type::isWildcard() const { return data()->isWildcard(); }

inline type::Flags Type::flags() const { return data()->flags(); }

inline bool Type::hasFlag(type::Flag f) const { return data()->hasFlag(std::move(f)); }

inline Type Type::iteratorType(bool const_) const { return data()->iteratorType(std::move(const_)); }

inline Type Type::viewType() const { return data()->viewType(); }

inline Type Type::dereferencedType() const { return data()->dereferencedType(); }

inline Type Type::elementType() const { return data()->elementType(); }

inline Type Type::effectiveType() const { return data()->effectiveType(); }

inline bool Type::_isConstant() const { return data()->_isConstant(); }

inline type::detail::State& Type::_state()  { return data()->_state(); }

inline std::optional<ID> Type::typeID() const { return data()->typeID(); }

inline std::optional<ID> Type::cxxID() const { return data()->cxxID(); }

inline hilti::node::Properties Type::properties() const { return data()->properties(); }

inline const std::vector<hilti::Node>& Type::childs() const { return data()->childs(); }

inline std::vector<hilti::Node>& Type::childs()  { return data()->childs(); }

inline const Meta& Type::meta() const { return data()->meta(); }

inline void Type::setMeta(Meta m)  { return data()->setMeta(std::move(m)); }

inline const NodeRef& Type::originalNode() const { return data()->originalNode(); }

inline void Type::setOriginalNode(const NodeRef& n)  { return data()->setOriginalNode(n); }

inline void Type::clearCache()  { return data()->clearCache(); }

inline bool Type::_isAllocable() const { return data()->_isAllocable(); }

inline bool Type::_isDereferencable() const { return data()->_isDereferencable(); }

inline bool Type::_isIterable() const { return data()->_isIterable(); }

inline bool Type::_isViewable() const { return data()->_isViewable(); }

inline bool Type::_isIterator() const { return data()->_isIterator(); }

inline bool Type::_isView() const { return data()->_isView(); }

inline bool Type::_isParameterized() const { return data()->_isParameterized(); }

inline bool Type::_isReferenceType() const { return data()->_isReferenceType(); }

inline bool Type::_isMutable() const { return data()->_isMutable(); }

inline bool Type::_hasDynamicType() const { return data()->_hasDynamicType(); }

inline bool Type::_isRuntimeNonTrivial() const { return data()->_isRuntimeNonTrivial(); }

inline bool Type::_isOnHeap() const { return data()->_isOnHeap(); }

inline Type Type::_clone() const { return data() ? data()->_clone() : Type(); }

inline hilti::IntrusivePtr<Concept> Type::_clone_ptr() const {
    if ( data() )
        return data()->_clone_ptr();
    else
        return nullptr;
}



