
class Item;

class Concept : public hilti::util::type_erasure::ConceptBase {
public:
    Concept() = default;
    Concept(const Concept&) = default;
    Concept(Concept&&) = default;
    virtual ~Concept() = default;
    /** Returns the type of the parsed unit item. */
    virtual Type itemType() const = 0;
    /** Returns true if the unit item is equivalent to another one. */
    virtual bool isEqual(const Item& other) const = 0;
    /** Implements the `Node` interface. */
    virtual hilti::node::Properties properties() const = 0;
    /** Implements the `Node` interface. */
    virtual const std::vector<hilti::Node>& childs() const = 0;
    /** Implements the `Node` interface. */
    virtual std::vector<hilti::Node>& childs()  = 0;
    /** Implements the `Node` interface. */
    virtual const Meta& meta() const = 0;
    /** Implements the `Node` interface. */
    virtual void setMeta(Meta m)  = 0;
    /** Implements the `Node` interface. */
    virtual const NodeRef& originalNode() const = 0;
    /** Implements the `Node` interface. */
    virtual void setOriginalNode(const NodeRef& n)  = 0;
    /** Implements the `Node` interface. */
    virtual void clearCache()  = 0;
    Concept& operator=(const Concept&) = default;
    Concept& operator=(Concept&&) = default;
    virtual Item _clone() const = 0;
    virtual hilti::IntrusivePtr<Concept> _clone_ptr() const = 0;
private:
    
};

template<typename T>
using ModelBase = hilti::util::type_erasure::ModelBase<T, Concept>;

template<class T>
class Model : public ModelBase<T> {
public:
    using ModelBase<T>::ModelBase;
    using ModelBase<T>::data;

    Type itemType() const final;
    bool isEqual(const Item& other) const final;
    hilti::node::Properties properties() const final;
    const std::vector<hilti::Node>& childs() const final;
    std::vector<hilti::Node>& childs()  final;
    const Meta& meta() const final;
    void setMeta(Meta m)  final;
    const NodeRef& originalNode() const final;
    void setOriginalNode(const NodeRef& n)  final;
    void clearCache()  final;
    Item _clone() const final;
    hilti::IntrusivePtr<Concept> _clone_ptr() const final;
};

using ErasedBase = hilti::util::type_erasure::ErasedBase<trait::isUnitItem, Concept, Model>;

class Item : public ErasedBase, public hilti::trait::isNode {
public:
    using ErasedBase::ErasedBase;

    Item() noexcept = default;
    Item(Item&&) noexcept = default;
    Item& operator=(Item&&) = default;
    ~Item() override = default;
    Item(const Item& other) = default;
    Item& operator=(const Item& other) = default;

    Type itemType() const;
    bool isEqual(const Item& other) const;
    hilti::node::Properties properties() const;
    const std::vector<hilti::Node>& childs() const;
    std::vector<hilti::Node>& childs() ;
    const Meta& meta() const;
    void setMeta(Meta m) ;
    const NodeRef& originalNode() const;
    void setOriginalNode(const NodeRef& n) ;
    void clearCache() ;
    Item _clone() const;

    hilti::IntrusivePtr<Concept> _clone_ptr() const;
};

template<typename T>
Type Model<T>::itemType() const { return data().itemType(); }

template<typename T>
bool Model<T>::isEqual(const Item& other) const { return data().isEqual(other); }

template<typename T>
hilti::node::Properties Model<T>::properties() const { return data().properties(); }

template<typename T>
const std::vector<hilti::Node>& Model<T>::childs() const { return data().childs(); }

template<typename T>
std::vector<hilti::Node>& Model<T>::childs()  { return data().childs(); }

template<typename T>
const Meta& Model<T>::meta() const { return data().meta(); }

template<typename T>
void Model<T>::setMeta(Meta m)  { return data().setMeta(m); }

template<typename T>
const NodeRef& Model<T>::originalNode() const { return data().originalNode(); }

template<typename T>
void Model<T>::setOriginalNode(const NodeRef& n)  { return data().setOriginalNode(n); }

template<typename T>
void Model<T>::clearCache()  { return data().clearCache(); }

template<typename T>
Item Model<T>::_clone() const { return T(data()); }

template<typename T>
hilti::IntrusivePtr<Concept> Model<T>::_clone_ptr() const {
    if constexpr ( std::is_base_of<hilti::util::type_erasure::trait::Singleton, T>::value )
        return nullptr;
    else
        return hilti::make_intrusive<Model>(T(data()));
}

inline Type Item::itemType() const { return data()->itemType(); }

inline bool Item::isEqual(const Item& other) const { return data()->isEqual(other); }

inline hilti::node::Properties Item::properties() const { return data()->properties(); }

inline const std::vector<hilti::Node>& Item::childs() const { return data()->childs(); }

inline std::vector<hilti::Node>& Item::childs()  { return data()->childs(); }

inline const Meta& Item::meta() const { return data()->meta(); }

inline void Item::setMeta(Meta m)  { return data()->setMeta(std::move(m)); }

inline const NodeRef& Item::originalNode() const { return data()->originalNode(); }

inline void Item::setOriginalNode(const NodeRef& n)  { return data()->setOriginalNode(n); }

inline void Item::clearCache()  { return data()->clearCache(); }

inline Item Item::_clone() const { return data() ? data()->_clone() : Item(); }

inline hilti::IntrusivePtr<Concept> Item::_clone_ptr() const {
    if ( data() )
        return data()->_clone_ptr();
    else
        return nullptr;
}



