// See the file "COPYING" in the main distribution directory for copyright.
#ifndef __SPCAPBUFF__H__
#define __SPCAPBUFF__H__
#include <sys/types.h>
#include <string>
#include <cstdio>
#include <queue>
#include <mutex>
#include <arpa/inet.h>
#include <sys/un.h>

#include "zeek/iosource/Packet.h"
#include "zeek/iosource/IOSource.h"
#include "zeek/threading/SerialTypes.h"
#include "zeek/threading/MsgThread.h"
#include "zeek/iosource/PktDumper.h"
#include "zeek/UID.h"
#include "BuffPool.h"

namespace SPcap {
//Used for writing files directly from zeek.
enum SPCmd {
	NONE,
	OPEN,
	WRITE,
	DISCARD,
	CLOSE,
	FFLUSH
};

//Actual buffer where packets are stored.
//This is streamed to ESM/file writer
struct PktMsg {
	uint32_t magicNumber;	// 'spcp' - could indicate version, or allow other packet types on the connection
	uint32_t curSize;	// payload size
	SPCmd cmd;
	uint32_t numPkts;// Number of packets stored
	u_char uidStr[32];//Base62 UID for current flow.
	uint8_t buffers[0];		// A sequence of PcapPacket
}	__attribute__ ((packed));

#define PKTMSG_HDR_SIZE  (sizeof(struct PktMsg))

//Overhead for each packet captured from wire.
#define MSG_OVERHEAD (sizeof(struct pcap_pkthdr))

//Size required to store one or more packet messages with libpcap header.
#define MSG_BUFFER_SIZE(sz) ((sz) + MSG_OVERHEAD + PKTMSG_HDR_SIZE)

#define MAX_TIME_ID_LENGTH (16)



struct PacketBuff {
	PacketBuff(size_t bufferSize);
	PacketBuff() = delete;
	~PacketBuff();

	bool AddPacket(const zeek::Packet *pkt); //Add packet to buffer
	void Init();
	void Init(zeek::UID id, SPCmd c);
	void Init(std::string tid, SPCmd c);

	zeek::UID uid;
	std::string tid;
	PktMsg *pmsg;//Points to rawBuffer

	uint8_t  *newPktMsg(size_t size);

	void AttachBuffer(size_t bufferSize);
	//append pcap header and packet to buffer
	void Append(struct pcap_pkthdr *phdr, const u_char *buff);

	//Reduce the size of pmsg
	//This limits how much data is stored.
	void TrimBuffer(uint64_t new_len);

	size_t Size() const
		{
        	return pmsg->curSize;
 		}

	size_t& Max()
		{
        	return pmsgBufferSize;
		}

	void Reset();//reset variables.
	
	//Sets the internal buffer size to newSize, optionally copying current contents
	//NOTE: if newSize < PayloadSize() data will be lost\truncated
	void Reset(size_t newSize, bool doCopy = false);
	size_t PayloadSize()
		{
		return rawPacketSize;
		}
  /*Check whether there is enough 
	free space to add pkt.*/
	bool SpaceAvailabe(const zeek::Packet *pkt) const;
	private:
	size_t pmsgBufferSize;
	size_t rawPacketSize;
	uint8_t  *rawBuffer;
};
}
#endif
