// See the file "COPYING" in the main distribution directory for copyright.
#ifndef __SPCAPFILEWRITER__H__
#define __SPCAPFILEWRITER__H__

#include <sys/types.h>
#include <string>
#include <cstdio>

#include "zeek/iosource/Packet.h"
#include "zeek/iosource/IOSource.h"
#include "zeek/iosource/PktDumper.h"
#include "zeek/UID.h"
#include "ThreadedWriter.h"

extern "C" {
#include <pcap.h>
}

//Writer for writing uid.pcaps


namespace SPcap {
class File{
public:
	File():is_open(false){}
	File(zeek::UID uid, std::string dir);
	virtual ~File();
	bool Write(std::unique_ptr<PacketBuff> &pb);
	bool Open(zeek::UID uid, std::string dir);
	bool Open(std::string dir);
	bool Close();

private:
	std::string id;
	bool is_open;
	std::string path;
	pcap_dumper_t *dumper;
	pcap_t *pf;
	std::string GetConnID(zeek::UID &uid);
};


/*Handle file pointers and invoke writes*/
//Implement writer 
class FileWriter: public ThreadedWriter  {
	public:
	FileWriter(uint64_t bufferSize, std::string dir): ThreadedWriter { bufferSize }
		{
		SetName("SmartPcapWriter");
		Start();
		SendResumeMessage();
		pcap_dir = dir;
		}

	ssize_t DoWrite(std::unique_ptr<PacketBuff> pb) override;
private:
	//DS for tracking open files.
	std::map<std::string, std::unique_ptr<File> > FileMap;
	std::string pcap_dir;
};


}
#endif
